"""Tests for plugin and/or channel specific tasks"""

import os
import json
import unittest
from types import SimpleNamespace
from plugins.channels import channels
from plugins.filters import PluginFilters
from plugins.filters import PluginFiltersError
from plugins import CHANNEL_ROOT_CACHE
from user.user import create_json_file, empty_json_file


class TestChannelRootEnvVarReplacement(unittest.TestCase):

    channel = "Pirates"
    launcher = SimpleNamespace(**{"hpc_variables": {
        "$HOME": "/p/home/jsparrow",
        "$PROJECTS_HOME": "/p/projects",
        "$WORKDIR": "/p/work1/jsparrow"
    }}) # A gimmicked InterfaceLauncher

    def test_basic_home_replacement(self):
        channel_root = "/p/home/jsparrow/test/black-pearl-plugin-channel"
        replaced = "$HOME/test/black-pearl-plugin-channel"
        assert replaced == channels.update_channel_root(self.launcher, channel_root, self.channel)

    def test_basic_projects_home_replacement(self):
        channel_root = "/p/projects/black_pearl/black-pearl-plugin-channel"
        replaced = "$PROJECTS_HOME/black_pearl/black-pearl-plugin-channel"
        assert replaced == channels.update_channel_root(self.launcher, channel_root, self.channel)

    def test_basic_workdir_replacement(self):
        channel_root = "/p/work1/jsparrow/black-pearl-plugin-channel"
        replaced = "$WORKDIR/black-pearl-plugin-channel"
        assert replaced == channels.update_channel_root(self.launcher, channel_root, self.channel)

    def test_multi_replace_issue(self):
        # /p/home/jsparrow should not be replaced
        channel_root = "/p/projects/p/home/jsparrow/black_pearl/black-pearl-plugin-channel"
        replaced = "$PROJECTS_HOME/p/home/jsparrow/black_pearl/black-pearl-plugin-channel"
        assert replaced == channels.update_channel_root(self.launcher, channel_root, self.channel)

    def test_no_replacement(self):
        channel_root = "/tmp/black_pearl/black-pearl-plugin-channel"
        replaced = "/tmp/black_pearl/black-pearl-plugin-channel"
        assert replaced == channels.update_channel_root(self.launcher, channel_root, self.channel)

class TestChannelRootCacheFile(unittest.TestCase):

    cache = "/tmp/channel_root_cache.json"

    @classmethod
    def setUpClass(cls):
        channels.CHANNEL_ROOT_CACHE = cls.cache
        create_json_file(cls.cache)

    @classmethod
    def tearDownClass(cls):
        channels.CHANNEL_ROOT_CACHE = CHANNEL_ROOT_CACHE
        os.remove(cls.cache)

    def setUp(self):
        empty_json_file(self.cache)

    def test_write_cache(self):
        channels.write_channel_root_to_cache("test", "$HOME/test")
        assert channels.get_channel_root_from_cache("test") == "$HOME/test"

    def test_update_existing_channel(self):
        self.test_write_cache()
        channels.write_channel_root_to_cache("test", "$HOME/test2")
        assert channels.get_channel_root_from_cache("test") == "$HOME/test2"

    def test_no_item_in_cache(self):
        assert channels.get_channel_root_from_cache("test") == ""

    def test_remove_item_from_cache(self):
        self.test_write_cache()
        channels.remove_channel_root_from_cache("test")
        assert channels.get_channel_root_from_cache("test") == ""

class TestFilter(unittest.TestCase):

    # Currenlty the PluginFilters' methods only require a plugin to have the "title" and "available-on" properties.
    # The above plugins.json file only contains the bare minimum data properties for these tests.
    test_dir = os.path.dirname(os.path.abspath(__file__))
    test_plugins_file = os.path.join(test_dir, "plugins/plugins.json")
    test_plugins_data = {}
    test_systems_file = os.path.join(test_dir, "systems/systems.json")
    test_systems_data = {}
    test_groups_file = os.path.join(test_dir, "systems/groups.json")
    test_groups_data = {}

    @classmethod
    def setUpClass(cls):
        # Load test plugins data
        with open(cls.test_plugins_file, 'r') as file:
            cls.test_plugins_data = json.load(file)
        # Load test systems data
        with open(cls.test_systems_file, 'r') as file:
            cls.test_systems_data = json.load(file)
        # Load test groups data
        with open(cls.test_groups_file, 'r') as file:
            cls.test_groups_data = json.load(file)

    def get_filter(self, plugins=None, system=None, node_type=None) -> PluginFilters:
        if not system:
            system = "DummySystem"
        if not node_type:
            node_type = "DummyNode"
        if not plugins:
            plugins = self.test_plugins_data
        f = PluginFilters(plugins, system, node_type)
        f.all_systems = self.test_systems_data
        f.all_groups = self.test_groups_data
        return f

    def test_get_system(self):
        f = self.get_filter()
        assert f._get_system("Raider") == "Raider"
        assert f._get_system("Raider:cpu:gpu") == "Raider"

    def test_get_group(self):
        f = self.get_filter()
        assert f._get_group("Raider") == None
        assert f._get_group("Raider:cpu:gpu") == None
        assert f._get_group("HPC-Group-x86") == "x86"
        assert f._get_group("HPC-Group-x86:cpu:gpu:another-node-type:abc123") == "x86"

    def test_is_system_match(self):
        f = self.get_filter()
        assert f._is_system_match("Narwhal") == False
        assert f._is_system_match("DummySystem") == True
        # "CustomSystem" exists within dummy system data with an "available_on_map"
        f = self.get_filter(system="CustomSystem")
        assert f._is_system_match("DummySystem") == False
        assert f._is_system_match("Raider") == True
        assert f._is_system_match("AnalyticsGateway") == True

    def test_is_group_match(self):
        f = self.get_filter(system="Raider")
        assert f._is_group_match("All") == True
        assert f._is_group_match("x86") == True
        assert f._is_group_match("ppcle") == False
        # "CustomSystem" exists within dummy system data with an "available_on_map"
        f = self.get_filter(system="CustomSystem")
        assert f._is_group_match("x86") == True
        assert f._is_group_match("AFRL") == False

    def test_is_node_type_match(self):
        f = self.get_filter()
        assert f._is_node_type_match([]) == True
        assert f._is_node_type_match(["n1", "n2", "DummyNode"]) == True
        assert f._is_node_type_match(["n1", "n2"]) == False
        f = self.get_filter(node_type="login")
        assert f._is_node_type_match([]) == False
        assert f._is_node_type_match(["n1", "login", "n2"]) == True
        assert f._is_node_type_match(["login"]) == True

    def test_get_node_types(self):
        f = self.get_filter()
        assert f._get_node_types("Raider") == []
        assert f._get_node_types("Raider:cpu:gpu") == ["cpu", "gpu"]
        assert f._get_node_types("HPC-Group-x86") == []
        assert f._get_node_types("HPC-Group-x86:gpu-2:gpu-4:gpu-8:mla+2:mla+10:mla+1:gpu_x86-64_A100-SXM4_8") == [
            "gpu-2",
            "gpu-4",
            "gpu-8",
            "mla+2",
            "mla+10",
            "mla+1",
            "gpu_x86-64_A100-SXM4_8"
        ]

    def test_get_channel_name(self):
        f = self.get_filter()
        assert f._get_channel_name("a::b") == "a"

    def test_get_plugin_name(self):
        f = self.get_filter()
        assert f._get_plugin_name("a::b") == "b"

    def test_is_system_AG(self):
        f = self.get_filter(system="Public_IT_Analytics_Gateway")
        assert f._is_system_AG() == True
        f = self.get_filter(system="Raider")
        assert f._is_system_AG() == False
        f = self.get_filter(system="CustomSystem")
        assert f._is_system_AG() == False
        f = self.get_filter(system="AnotherSystemThatShouldNotExist")
        assert f._is_system_AG() == False

    def test_is_plugin_AG(self):
        f = self.get_filter()
        assert f._is_plugin_AG("default::CodeServer") == False
        assert f._is_plugin_AG("Public_IT_Analytics_Gateway::CodeServer") == True

    def test_is_plugin_personal(self):
        f = self.get_filter()
        assert f._is_plugin_personal("default::CodeServer") == False
        assert f._is_plugin_personal("notpersonal::CodeServer") == False
        assert f._is_plugin_personal("personal::CodeServer") == True

    def test_get_available_on_map_systems(self):
        f = self.get_filter()
        assert f._get_available_on_map_systems() == []
        f = self.get_filter(system="CustomSystem")
        assert f._get_available_on_map_systems() == ["AnalyticsGateway", "Raider"]

    def test_get_available_on_map_groups(self):
        f = self.get_filter()
        assert f._get_available_on_map_groups() == []
        f = self.get_filter(system="CustomSystem")
        assert f._get_available_on_map_groups() == ["x86"]

    def test_sort_keys_by_plugin_title(self):
        f = self.get_filter()
        assert f._sort_keys_by_plugin_title(["air::air", "air::afsim_login"]) == ["air::afsim_login", "air::air"]

    def test_filter_by_available_on(self):
        f = self.get_filter(system="Raider", node_type="standard")
        expected = ["personal::CodeServer", "kasm-ubuntu::Ubuntu_Desktop", "air::air", "default::TTYD", "default::Jupyter", "default::Jupyter2"]
        assert sorted(f._filter_by_available_on()) == sorted(expected)
        f = self.get_filter(system="Raider", node_type="mla")
        expected = ["personal::CodeServer", "default::TTYD", "air::air", "default::Jupyter", "default::Jupyter2"]
        assert sorted(f._filter_by_available_on()) == sorted(expected)
        f = self.get_filter(system="Coral", node_type="login")
        expected = ["personal::TTYD"]
        assert sorted(f._filter_by_available_on()) == sorted(expected)

    def test_filter_by_analytics_gateway(self):
        f = self.get_filter(system="Public_IT_Analytics_Gateway", node_type=None)
        expected = ["personal::TTYD", "Public_IT_Analytics_Gateway::Jupyter"]
        assert sorted(f._filter_by_analytics_gateway()) == sorted(expected)

    def test_filter_raises_PluginFiltersError_for_invalid_system(self):
        f = self.get_filter(system="does_not_exixt")
        try:
            f.filter()
        except PluginFiltersError as e:
            if "valid system" in str(e):
                assert True
                return
        assert False

    def test_filter_raises_PluginFiltersError_for_invalid_node_type(self):
        f = self.get_filter(system="Raider", node_type="does_not_exixt")
        try:
            f.filter()
        except PluginFiltersError as e:
            if "valid node type" in str(e):
                assert True
                return
        assert False

    def test_fuzzy_search(self):
        f = self.get_filter()
        assert f._search("my plugin", ["a", "b", "c"]) == False
        assert f._search("my plugin", ["a", "myplugin", "c"]) == True
        assert f._search("Jupyter", ["a", "notjupyter", "c"]) == True
        assert f._search("Jupyer", ["Code Server", "Jupyter", "Kibana"]) == True
        assert f._search("Direct Match", ["not a good match", "maybe more direct"]) == False
        assert f._search("Direct Match", ["not a good match", "maybe more direct", "Direct Match"]) == True
        assert f._search("code", ["CodeServer"]) == True
        assert f._search("couch", ["CB+Couchbase+Java"]) == True

